searchData={"items":[{"type":"module","title":"LLMAgent.Application","doc":"The LLMAgent Application module.\nStarts the supervisor tree and manages the application lifecycle.","ref":"LLMAgent.Application.html"},{"type":"module","title":"LLMAgent","doc":"LLMAgent is an abstraction library for building domain-specific intelligent agents based on \nLarge Language Models (LLMs). It provides a core architecture and behavior definitions \nthat simplify the development of specialized agents.\n\nLLMAgent is built on top of AgentForge's signal-driven architecture and provides\nLLM-specific interaction patterns, signal types, message processing handlers,\ntool integration, and conversation management.","ref":"LLMAgent.html"},{"type":"module","title":"Key Features - LLMAgent","doc":"- LLM-specific signal types for agent interactions\n- Predefined handlers for processing LLM signals\n- Standard flow compositions for common agent patterns\n- Plugin-based extensions for LLM providers\n- Task management for long-running operations","ref":"LLMAgent.html#module-key-features"},{"type":"module","title":"Usage Example - LLMAgent","doc":"```elixir\n# Create agent with system prompt and basic tools\n{flow, initial_state} = LLMAgent.Flows.conversation(\n  \"You are a helpful assistant that can answer questions and use tools.\",\n  [\n    %{\n      name: \"search\",\n      description: \"Search the web for information\",\n      parameters: %{\n        \"type\" => \"object\",\n        \"properties\" => %{\n          \"query\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"Search query\"\n          }\n        },\n        \"required\" => [\"query\"]\n      },\n      execute: &MyApp.Tools.search/1\n    }\n  ]\n)\n\n# Process a user message\n{:ok, result, new_state} = AgentForge.process(\n  flow,\n  LLMAgent.Signals.user_message(\"What is the capital of France?\"),\n  initial_state\n)\n```","ref":"LLMAgent.html#module-usage-example"},{"type":"function","title":"LLMAgent.new/3","doc":"Creates a new LLM agent with the given system prompt and tools.\n\nThis is a convenience function that creates a flow and initial state for a conversational agent.","ref":"LLMAgent.html#new/3"},{"type":"function","title":"Parameters - LLMAgent.new/3","doc":"- `system_prompt` - The system prompt that defines the agent's behavior\n- `tools` - A list of tools that the agent can use\n- `options` - Additional options for configuring the agent","ref":"LLMAgent.html#new/3-parameters"},{"type":"function","title":"Returns - LLMAgent.new/3","doc":"A tuple containing the flow and initial state for the agent.","ref":"LLMAgent.html#new/3-returns"},{"type":"function","title":"Examples - LLMAgent.new/3","doc":"iex> {flow, state} = LLMAgent.new(\"You are a helpful assistant.\", [])\n    iex> is_function(flow) and is_map(state)\n    true","ref":"LLMAgent.html#new/3-examples"},{"type":"function","title":"LLMAgent.process/4","doc":"Processes a user message through an LLM agent flow.\n\nThis is a convenience function that creates a user message signal and processes it through the agent flow.","ref":"LLMAgent.html#process/4"},{"type":"function","title":"Parameters - LLMAgent.process/4","doc":"- `flow` - The agent flow to process the message through\n- `state` - The current state of the agent\n- `message` - The user message to process\n- `options` - Additional options for processing the message","ref":"LLMAgent.html#process/4-parameters"},{"type":"function","title":"Returns - LLMAgent.process/4","doc":"A tuple containing the result of processing the message and the new state.","ref":"LLMAgent.html#process/4-returns"},{"type":"function","title":"Examples - LLMAgent.process/4","doc":"iex> {flow, state} = LLMAgent.new(\"You are a helpful assistant.\", [])\n    iex> {:ok, result, _} = LLMAgent.process(flow, state, \"Hello\")\n    iex> result.type == :response\n    true","ref":"LLMAgent.html#process/4-examples"},{"type":"function","title":"LLMAgent.version/0","doc":"Returns the current version of LLMAgent.","ref":"LLMAgent.html#version/0"},{"type":"function","title":"Examples - LLMAgent.version/0","doc":"iex> LLMAgent.version()\n    \"3.0.0\"","ref":"LLMAgent.html#version/0-examples"},{"type":"module","title":"LLMAgent.Flows","doc":"Provides standard flow definitions for common LLM agent patterns.\n\nThis module creates AgentForge flow compositions for different LLM agent use cases,\nsuch as conversational agents and task-based agents. It registers tools, creates\nappropriate handlers, and configures initial state.","ref":"LLMAgent.Flows.html"},{"type":"function","title":"LLMAgent.Flows.batch_processing/3","doc":"Creates a batch processing flow.\n\nA batch processing flow iterates over a collection of items and applies\nthe batch handler to each item.","ref":"LLMAgent.Flows.html#batch_processing/3"},{"type":"function","title":"Parameters - LLMAgent.Flows.batch_processing/3","doc":"- `items` - The collection of items to process\n- `batch_handler` - The handler function to apply to each item\n- `options` - Additional options for batch processing","ref":"LLMAgent.Flows.html#batch_processing/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.batch_processing/3","doc":"A function that can be used as a flow.","ref":"LLMAgent.Flows.html#batch_processing/3-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.batch_processing/3","doc":"iex> items = [1, 2, 3]\n    iex> handler = fn signal, state -> {{:emit, signal}, state} end\n    iex> flow = LLMAgent.Flows.batch_processing(items, handler)\n    iex> is_function(flow)\n    true","ref":"LLMAgent.Flows.html#batch_processing/3-examples"},{"type":"function","title":"LLMAgent.Flows.conversation/3","doc":"Creates a standard conversation flow with the given system prompt and tools.","ref":"LLMAgent.Flows.html#conversation/3"},{"type":"function","title":"Parameters - LLMAgent.Flows.conversation/3","doc":"- `system_prompt` - The system prompt that defines the agent's behavior\n- `tools` - A list of tools that the agent can use\n- `options` - Additional options for configuring the agent","ref":"LLMAgent.Flows.html#conversation/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.conversation/3","doc":"A tuple containing the flow and initial state for the agent.","ref":"LLMAgent.Flows.html#conversation/3-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.conversation/3","doc":"iex> {flow, state} = LLMAgent.Flows.conversation(\"You are a helpful assistant.\", [])\n    iex> is_function(flow) and is_map(state)\n    true","ref":"LLMAgent.Flows.html#conversation/3-examples"},{"type":"function","title":"LLMAgent.Flows.map_flow/2","doc":"Maps a flow to a new flow by applying a transformation function.\n\nUseful for creating derived flows that modify signal or state handling.","ref":"LLMAgent.Flows.html#map_flow/2"},{"type":"function","title":"Parameters - LLMAgent.Flows.map_flow/2","doc":"- `flow` - The original flow\n- `transform_fn` - A function that transforms the flow's result","ref":"LLMAgent.Flows.html#map_flow/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.map_flow/2","doc":"A new flow that applies the transformation.","ref":"LLMAgent.Flows.html#map_flow/2-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.map_flow/2","doc":"iex> original_flow = fn signal, state -> {{:emit, signal}, state} end\n    iex> transform = fn {result, state} -> {result, Map.put(state, :transformed, true)} end\n    iex> new_flow = LLMAgent.Flows.map_flow(original_flow, transform)\n    iex> is_function(new_flow)\n    true","ref":"LLMAgent.Flows.html#map_flow/2-examples"},{"type":"function","title":"LLMAgent.Flows.qa_agent/2","doc":"Creates a flow for a simple question-answering agent.\n\nThis is a specialized flow for handling basic question-answering without tools,\nproviding a streamlined implementation for simple use cases.","ref":"LLMAgent.Flows.html#qa_agent/2"},{"type":"function","title":"Parameters - LLMAgent.Flows.qa_agent/2","doc":"- `system_prompt` - The system prompt that defines the agent's behavior\n- `options` - Additional options for configuring the agent","ref":"LLMAgent.Flows.html#qa_agent/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.qa_agent/2","doc":"A tuple containing the flow and initial state for the agent.","ref":"LLMAgent.Flows.html#qa_agent/2-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.qa_agent/2","doc":"iex> {flow, state} = LLMAgent.Flows.qa_agent(\"You are a helpful assistant.\")\n    iex> is_function(flow) and is_map(state)\n    true","ref":"LLMAgent.Flows.html#qa_agent/2-examples"},{"type":"function","title":"LLMAgent.Flows.task_flow/2","doc":"Creates a task flow with the given task definition.\n\nA task flow executes a series of primitive operations defined in the task definition.","ref":"LLMAgent.Flows.html#task_flow/2"},{"type":"function","title":"Parameters - LLMAgent.Flows.task_flow/2","doc":"- `task_definition` - The task definition as a list of AgentForge primitives\n- `options` - Additional options for the task flow","ref":"LLMAgent.Flows.html#task_flow/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.task_flow/2","doc":"A function that can be used as a flow.","ref":"LLMAgent.Flows.html#task_flow/2-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.task_flow/2","doc":"iex> task_def = [\n    ...>   fn data -> Map.put(data, :step1, true) end\n    ...> ]\n    iex> flow = LLMAgent.Flows.task_flow(task_def)\n    iex> is_function(flow)\n    true","ref":"LLMAgent.Flows.html#task_flow/2-examples"},{"type":"function","title":"LLMAgent.Flows.tool_agent/3","doc":"Creates a flow for a tool-using agent.","ref":"LLMAgent.Flows.html#tool_agent/3"},{"type":"function","title":"Parameters - LLMAgent.Flows.tool_agent/3","doc":"- `system_prompt` - The system prompt that defines the agent's behavior\n- `tools` - A list of tools that the agent can use\n- `options` - Additional options for configuring the agent","ref":"LLMAgent.Flows.html#tool_agent/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.tool_agent/3","doc":"A tuple containing the flow and initial state for the agent.","ref":"LLMAgent.Flows.html#tool_agent/3-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.tool_agent/3","doc":"iex> tools = [\n    ...>   %{name: \"get_time\", description: \"Get the current time\", execute: fn _ -> %{time: DateTime.utc_now()} end}\n    ...> ]\n    iex> {flow, state} = LLMAgent.Flows.tool_agent(\"You are a helpful assistant that can use tools.\", tools)\n    iex> is_function(flow) and is_map(state)\n    true","ref":"LLMAgent.Flows.html#tool_agent/3-examples"},{"type":"function","title":"LLMAgent.Flows.with_middleware/2","doc":"Adds middleware to a flow.\n\nMiddleware functions are executed before and after the main flow,\nallowing for consistent pre- and post-processing.","ref":"LLMAgent.Flows.html#with_middleware/2"},{"type":"function","title":"Parameters - LLMAgent.Flows.with_middleware/2","doc":"- `flow` - The original flow\n- `middleware` - A function that receives the signal, state, and a continuation","ref":"LLMAgent.Flows.html#with_middleware/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Flows.with_middleware/2","doc":"A new flow with the middleware applied.","ref":"LLMAgent.Flows.html#with_middleware/2-returns"},{"type":"function","title":"Examples - LLMAgent.Flows.with_middleware/2","doc":"iex> flow = fn signal, state -> {{:emit, signal}, state} end\n    iex> middleware = fn signal, state, continue ->\n    ...>   {result, new_state} = continue.(signal, state)\n    ...>   {result, Map.put(new_state, :middleware_applied, true)}\n    ...> end\n    iex> new_flow = LLMAgent.Flows.with_middleware(flow, middleware)\n    iex> is_function(new_flow)\n    true","ref":"LLMAgent.Flows.html#with_middleware/2-examples"},{"type":"module","title":"LLMAgent.Handlers","doc":"Provides standard handlers for processing LLM-specific signals.\n\nThis module implements signal handlers for the various LLM agent signal types,\nfollowing AgentForge's handler pattern. Each handler takes a signal and state,\nprocesses the signal, and returns a tuple with a result and a new state.","ref":"LLMAgent.Handlers.html"},{"type":"function","title":"LLMAgent.Handlers.error_handler/2","doc":"Handles error signals.\n\nProcesses errors and may implement recovery strategies.","ref":"LLMAgent.Handlers.html#error_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.error_handler/2","doc":"- `signal` - The error signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#error_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.error_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#error_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.error_handler/2","doc":"iex> signal = LLMAgent.Signals.error(\"API unavailable\", \"get_weather\")\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.error_handler(signal, state)\n    iex> is_tuple(result) and is_map(new_state)\n    true","ref":"LLMAgent.Handlers.html#error_handler/2-examples"},{"type":"function","title":"LLMAgent.Handlers.message_handler/2","doc":"Handles user message signals.\n\nTakes a user message, adds it to history, and generates the appropriate next signal\n(thinking, tool call, or response) based on the message content and available tools.","ref":"LLMAgent.Handlers.html#message_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.message_handler/2","doc":"- `signal` - The user message signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#message_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.message_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#message_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.message_handler/2","doc":"iex> signal = LLMAgent.Signals.user_message(\"Hello\")\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.message_handler(signal, state)\n    iex> match?({:emit, %{type: :thinking}}, result) or match?({:emit, %{type: :response}}, result)\n    true","ref":"LLMAgent.Handlers.html#message_handler/2-examples"},{"type":"function","title":"LLMAgent.Handlers.response_handler/2","doc":"Handles response signals.\n\nFormats the final response and may trigger notifications or other side effects.","ref":"LLMAgent.Handlers.html#response_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.response_handler/2","doc":"- `signal` - The response signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#response_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.response_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#response_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.response_handler/2","doc":"iex> signal = LLMAgent.Signals.response(\"AAPL is trading at $200\")\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.response_handler(signal, state)\n    iex> is_tuple(result) and is_map(new_state)\n    true","ref":"LLMAgent.Handlers.html#response_handler/2-examples"},{"type":"function","title":"LLMAgent.Handlers.task_handler/2","doc":"Handles task state signals.\n\nUpdates the state of a task and generates a notification if necessary.","ref":"LLMAgent.Handlers.html#task_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.task_handler/2","doc":"- `signal` - The task state signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#task_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.task_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#task_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.task_handler/2","doc":"iex> signal = LLMAgent.Signals.task_state(\"task_123\", \"completed\")\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.task_handler(signal, state)\n    iex> is_tuple(result) and is_map(new_state)\n    true","ref":"LLMAgent.Handlers.html#task_handler/2-examples"},{"type":"function","title":"LLMAgent.Handlers.thinking_handler/2","doc":"Handles thinking step signals.\n\nProcesses a thinking step, adds it to state, and determines the next action\n(continue thinking, call a tool, or generate a response).","ref":"LLMAgent.Handlers.html#thinking_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.thinking_handler/2","doc":"- `signal` - The thinking signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#thinking_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.thinking_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#thinking_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.thinking_handler/2","doc":"iex> signal = LLMAgent.Signals.thinking(\"I need stock data\", 1)\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.thinking_handler(signal, state)\n    iex> is_tuple(result) and is_map(new_state)\n    true","ref":"LLMAgent.Handlers.html#thinking_handler/2-examples"},{"type":"function","title":"LLMAgent.Handlers.tool_handler/2","doc":"Handles tool call signals.\n\nExecutes the specified tool with the provided arguments and generates a tool result signal.","ref":"LLMAgent.Handlers.html#tool_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.tool_handler/2","doc":"- `signal` - The tool call signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#tool_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.tool_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#tool_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.tool_handler/2","doc":"iex> signal = LLMAgent.Signals.tool_call(\"get_weather\", %{city: \"New York\"})\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.tool_handler(signal, state)\n    iex> is_tuple(result) and is_map(new_state)\n    true","ref":"LLMAgent.Handlers.html#tool_handler/2-examples"},{"type":"function","title":"LLMAgent.Handlers.tool_result_handler/2","doc":"Handles tool result signals.\n\nProcesses the result of a tool execution and generates the next signal based on the result.","ref":"LLMAgent.Handlers.html#tool_result_handler/2"},{"type":"function","title":"Parameters - LLMAgent.Handlers.tool_result_handler/2","doc":"- `signal` - The tool result signal\n- `state` - The current store state","ref":"LLMAgent.Handlers.html#tool_result_handler/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Handlers.tool_result_handler/2","doc":"A tuple containing the result and updated state.","ref":"LLMAgent.Handlers.html#tool_result_handler/2-returns"},{"type":"function","title":"Examples - LLMAgent.Handlers.tool_result_handler/2","doc":"iex> signal = LLMAgent.Signals.tool_result(\"get_weather\", %{temp: 72})\n    iex> state = LLMAgent.Store.new()\n    iex> {result, new_state} = LLMAgent.Handlers.tool_result_handler(signal, state)\n    iex> is_tuple(result) and is_map(new_state)\n    true","ref":"LLMAgent.Handlers.html#tool_result_handler/2-examples"},{"type":"module","title":"LLMAgent.Signals","doc":"Defines LLM-specific signal types and helpers for agent communication.\n\nThis module extends AgentForge.Signal with specialized signal types for LLM interactions,\nproviding a consistent interface for creating signals that represent different stages\nof LLM agent processing.","ref":"LLMAgent.Signals.html"},{"type":"module","title":"Signal Types - LLMAgent.Signals","doc":"- `:user_message` - A message from the user\n- `:system_message` - A system message\n- `:thinking` - An agent thinking step\n- `:tool_call` - A tool call request\n- `:tool_result` - A tool execution result\n- `:task_state` - A task state update\n- `:response` - An agent response\n- `:error` - An error signal","ref":"LLMAgent.Signals.html#module-signal-types"},{"type":"function","title":"LLMAgent.Signals.error/3","doc":"Creates an error signal.","ref":"LLMAgent.Signals.html#error/3"},{"type":"function","title":"Parameters - LLMAgent.Signals.error/3","doc":"- `message` - The error message\n- `source` - The source of the error\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#error/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.error/3","doc":"A signal of type `:error` with the provided message, source, and metadata.","ref":"LLMAgent.Signals.html#error/3-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.error/3","doc":"iex> signal = LLMAgent.Signals.error(\"API unavailable\", \"get_stock_price\")\n    iex> signal.type == :error\n    true\n    iex> signal.data.message == \"API unavailable\" and signal.data.source == \"get_stock_price\"\n    true","ref":"LLMAgent.Signals.html#error/3-examples"},{"type":"function","title":"LLMAgent.Signals.response/2","doc":"Creates a response signal.","ref":"LLMAgent.Signals.html#response/2"},{"type":"function","title":"Parameters - LLMAgent.Signals.response/2","doc":"- `content` - The content of the response\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#response/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.response/2","doc":"A signal of type `:response` with the provided content and metadata.","ref":"LLMAgent.Signals.html#response/2-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.response/2","doc":"iex> signal = LLMAgent.Signals.response(\"AAPL is trading at $200\")\n    iex> signal.type == :response and signal.data == \"AAPL is trading at $200\"\n    true","ref":"LLMAgent.Signals.html#response/2-examples"},{"type":"function","title":"LLMAgent.Signals.system_message/2","doc":"Creates a system message signal.","ref":"LLMAgent.Signals.html#system_message/2"},{"type":"function","title":"Parameters - LLMAgent.Signals.system_message/2","doc":"- `content` - The content of the system message\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#system_message/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.system_message/2","doc":"A signal of type `:system_message` with the provided content and metadata.","ref":"LLMAgent.Signals.html#system_message/2-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.system_message/2","doc":"iex> signal = LLMAgent.Signals.system_message(\"You are a helpful assistant.\")\n    iex> signal.type == :system_message and signal.data == \"You are a helpful assistant.\"\n    true","ref":"LLMAgent.Signals.html#system_message/2-examples"},{"type":"function","title":"LLMAgent.Signals.task_state/3","doc":"Creates a task state signal.","ref":"LLMAgent.Signals.html#task_state/3"},{"type":"function","title":"Parameters - LLMAgent.Signals.task_state/3","doc":"- `task_id` - The ID of the task\n- `state` - The current state of the task\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#task_state/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.task_state/3","doc":"A signal of type `:task_state` with the provided task ID, state, and metadata.","ref":"LLMAgent.Signals.html#task_state/3-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.task_state/3","doc":"iex> signal = LLMAgent.Signals.task_state(\"task_123\", \"running\")\n    iex> signal.type == :task_state\n    true\n    iex> signal.data.task_id == \"task_123\" and signal.data.state == \"running\"\n    true","ref":"LLMAgent.Signals.html#task_state/3-examples"},{"type":"function","title":"LLMAgent.Signals.thinking/3","doc":"Creates a thinking signal.","ref":"LLMAgent.Signals.html#thinking/3"},{"type":"function","title":"Parameters - LLMAgent.Signals.thinking/3","doc":"- `thought` - The content of the thinking step\n- `step` - The step number in the thinking process\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#thinking/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.thinking/3","doc":"A signal of type `:thinking` with the provided thought and metadata including step.","ref":"LLMAgent.Signals.html#thinking/3-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.thinking/3","doc":"iex> signal = LLMAgent.Signals.thinking(\"I need to get stock data\", 1)\n    iex> signal.type == :thinking and signal.data == \"I need to get stock data\"\n    true\n    iex> signal.meta.step\n    1","ref":"LLMAgent.Signals.html#thinking/3-examples"},{"type":"function","title":"LLMAgent.Signals.tool_call/3","doc":"Creates a tool call signal.","ref":"LLMAgent.Signals.html#tool_call/3"},{"type":"function","title":"Parameters - LLMAgent.Signals.tool_call/3","doc":"- `name` - The name of the tool to call\n- `args` - The arguments for the tool call\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#tool_call/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.tool_call/3","doc":"A signal of type `:tool_call` with the provided tool name, arguments, and metadata.","ref":"LLMAgent.Signals.html#tool_call/3-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.tool_call/3","doc":"iex> signal = LLMAgent.Signals.tool_call(\"get_stock_price\", %{ticker: \"AAPL\"})\n    iex> signal.type == :tool_call\n    true\n    iex> signal.data.name == \"get_stock_price\" and signal.data.args.ticker == \"AAPL\"\n    true","ref":"LLMAgent.Signals.html#tool_call/3-examples"},{"type":"function","title":"LLMAgent.Signals.tool_result/3","doc":"Creates a tool result signal.","ref":"LLMAgent.Signals.html#tool_result/3"},{"type":"function","title":"Parameters - LLMAgent.Signals.tool_result/3","doc":"- `name` - The name of the tool that was called\n- `result` - The result of the tool execution\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#tool_result/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.tool_result/3","doc":"A signal of type `:tool_result` with the provided tool name, result, and metadata.","ref":"LLMAgent.Signals.html#tool_result/3-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.tool_result/3","doc":"iex> signal = LLMAgent.Signals.tool_result(\"get_stock_price\", %{price: 200.50})\n    iex> signal.type == :tool_result\n    true\n    iex> signal.data.name == \"get_stock_price\" and signal.data.result.price == 200.50\n    true","ref":"LLMAgent.Signals.html#tool_result/3-examples"},{"type":"function","title":"LLMAgent.Signals.user_message/2","doc":"Creates a user message signal.","ref":"LLMAgent.Signals.html#user_message/2"},{"type":"function","title":"Parameters - LLMAgent.Signals.user_message/2","doc":"- `content` - The content of the user message\n- `meta` - Additional metadata for the signal","ref":"LLMAgent.Signals.html#user_message/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Signals.user_message/2","doc":"A signal of type `:user_message` with the provided content and metadata.","ref":"LLMAgent.Signals.html#user_message/2-returns"},{"type":"function","title":"Examples - LLMAgent.Signals.user_message/2","doc":"iex> signal = LLMAgent.Signals.user_message(\"Hello\")\n    iex> signal.type == :user_message and signal.data == \"Hello\"\n    true\n    \n    iex> signal = LLMAgent.Signals.user_message(\"Hello\", %{user_id: \"123\"})\n    iex> signal.meta.user_id\n    \"123\"","ref":"LLMAgent.Signals.html#user_message/2-examples"},{"type":"module","title":"LLMAgent.Store","doc":"Manages state for LLM agent conversations.\n\nThis module provides helpers for working with the LLM agent state, including\nmanaging conversation history, thoughts, tool calls, and tasks. It follows\nAgentForge's immutable state pattern.","ref":"LLMAgent.Store.html"},{"type":"function","title":"LLMAgent.Store.add_message/3","doc":"Adds a message to history.","ref":"LLMAgent.Store.html#add_message/3"},{"type":"function","title":"Parameters - LLMAgent.Store.add_message/3","doc":"- `state` - The current store state\n- `role` - The role of the message (e.g., \"user\", \"assistant\", \"system\")\n- `content` - The content of the message","ref":"LLMAgent.Store.html#add_message/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.add_message/3","doc":"An updated state with the new message added to history.","ref":"LLMAgent.Store.html#add_message/3-returns"},{"type":"function","title":"Examples - LLMAgent.Store.add_message/3","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.add_message(state, \"user\", \"Hello\")\n    iex> [%{role: \"user\", content: \"Hello\"}] = state.history","ref":"LLMAgent.Store.html#add_message/3-examples"},{"type":"function","title":"LLMAgent.Store.add_task/2","doc":"Adds a task to the current tasks list.","ref":"LLMAgent.Store.html#add_task/2"},{"type":"function","title":"Parameters - LLMAgent.Store.add_task/2","doc":"- `state` - The current store state\n- `task` - The task to add","ref":"LLMAgent.Store.html#add_task/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.add_task/2","doc":"An updated state with the new task added to the current tasks list.","ref":"LLMAgent.Store.html#add_task/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.add_task/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> task = %{id: \"task_123\", type: \"analysis\", status: \"running\"}\n    iex> state = LLMAgent.Store.add_task(state, task)\n    iex> [%{id: \"task_123\"}] = state.current_tasks","ref":"LLMAgent.Store.html#add_task/2-examples"},{"type":"function","title":"LLMAgent.Store.add_thought/2","doc":"Adds a thought to the current processing cycle.","ref":"LLMAgent.Store.html#add_thought/2"},{"type":"function","title":"Parameters - LLMAgent.Store.add_thought/2","doc":"- `state` - The current store state\n- `thought` - The thought content","ref":"LLMAgent.Store.html#add_thought/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.add_thought/2","doc":"An updated state with the new thought added to the thoughts list.","ref":"LLMAgent.Store.html#add_thought/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.add_thought/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.add_thought(state, \"I should look up stock prices\")\n    iex> [\"I should look up stock prices\"] = state.thoughts","ref":"LLMAgent.Store.html#add_thought/2-examples"},{"type":"function","title":"LLMAgent.Store.add_tool_call/4","doc":"Adds a tool call record to the state.","ref":"LLMAgent.Store.html#add_tool_call/4"},{"type":"function","title":"Parameters - LLMAgent.Store.add_tool_call/4","doc":"- `state` - The current store state\n- `name` - The name of the tool\n- `args` - The arguments passed to the tool\n- `result` - The result of the tool call","ref":"LLMAgent.Store.html#add_tool_call/4-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.add_tool_call/4","doc":"An updated state with the new tool call record added.","ref":"LLMAgent.Store.html#add_tool_call/4-returns"},{"type":"function","title":"Examples - LLMAgent.Store.add_tool_call/4","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.add_tool_call(state, \"get_weather\", %{city: \"New York\"}, %{temp: 72})\n    iex> [%{name: \"get_weather\", args: %{city: \"New York\"}, result: %{temp: 72}}] = state.tool_calls","ref":"LLMAgent.Store.html#add_tool_call/4-examples"},{"type":"function","title":"LLMAgent.Store.get_llm_history/2","doc":"Gets LLM history in the format expected by LLM providers.","ref":"LLMAgent.Store.html#get_llm_history/2"},{"type":"function","title":"Parameters - LLMAgent.Store.get_llm_history/2","doc":"- `state` - The current store state\n- `max_length` - The maximum number of history entries to return (default: 10)","ref":"LLMAgent.Store.html#get_llm_history/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.get_llm_history/2","doc":"A list of history entries, limited to the specified maximum length.","ref":"LLMAgent.Store.html#get_llm_history/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.get_llm_history/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.add_message(state, \"system\", \"You are a helpful assistant.\")\n    iex> state = LLMAgent.Store.add_message(state, \"user\", \"Hello\")\n    iex> history = LLMAgent.Store.get_llm_history(state)\n    iex> length(history) == 2\n    true","ref":"LLMAgent.Store.html#get_llm_history/2-examples"},{"type":"function","title":"LLMAgent.Store.get_preference/3","doc":"Gets a specific user preference.","ref":"LLMAgent.Store.html#get_preference/3"},{"type":"function","title":"Parameters - LLMAgent.Store.get_preference/3","doc":"- `state` - The current store state\n- `key` - The preference key to get\n- `default` - The default value to return if the preference is not set","ref":"LLMAgent.Store.html#get_preference/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.get_preference/3","doc":"The preference value or the default value.","ref":"LLMAgent.Store.html#get_preference/3-returns"},{"type":"function","title":"Examples - LLMAgent.Store.get_preference/3","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.set_preferences(state, %{language: \"en\"})\n    iex> LLMAgent.Store.get_preference(state, :language, \"fr\")\n    \"en\"\n    iex> LLMAgent.Store.get_preference(state, :theme, \"dark\")\n    \"dark\"","ref":"LLMAgent.Store.html#get_preference/3-examples"},{"type":"function","title":"LLMAgent.Store.get_preferences/1","doc":"Gets all preferences from the state.","ref":"LLMAgent.Store.html#get_preferences/1"},{"type":"function","title":"Parameters - LLMAgent.Store.get_preferences/1","doc":"- `state` - The current store state","ref":"LLMAgent.Store.html#get_preferences/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.get_preferences/1","doc":"A map of preferences.","ref":"LLMAgent.Store.html#get_preferences/1-returns"},{"type":"function","title":"Examples - LLMAgent.Store.get_preferences/1","doc":"iex> state = LLMAgent.Store.new(%{preferences: %{theme: \"dark\"}})\n    iex> LLMAgent.Store.get_preferences(state)\n    %{theme: \"dark\"}","ref":"LLMAgent.Store.html#get_preferences/1-examples"},{"type":"function","title":"LLMAgent.Store.get_thoughts/1","doc":"Gets all thoughts for the current processing cycle.","ref":"LLMAgent.Store.html#get_thoughts/1"},{"type":"function","title":"Parameters - LLMAgent.Store.get_thoughts/1","doc":"- `state` - The current store state","ref":"LLMAgent.Store.html#get_thoughts/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.get_thoughts/1","doc":"A list of thoughts from the current processing cycle.","ref":"LLMAgent.Store.html#get_thoughts/1-returns"},{"type":"function","title":"Examples - LLMAgent.Store.get_thoughts/1","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.add_thought(state, \"First thought\")\n    iex> state = LLMAgent.Store.add_thought(state, \"Second thought\")\n    iex> thoughts = LLMAgent.Store.get_thoughts(state)\n    iex> length(thoughts) == 2\n    true","ref":"LLMAgent.Store.html#get_thoughts/1-examples"},{"type":"function","title":"LLMAgent.Store.new/1","doc":"Creates a new store with default values.","ref":"LLMAgent.Store.html#new/1"},{"type":"function","title":"Parameters - LLMAgent.Store.new/1","doc":"- `attrs` - A map of attributes to merge with the default store","ref":"LLMAgent.Store.html#new/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.new/1","doc":"A new store map with default values merged with the provided attributes.","ref":"LLMAgent.Store.html#new/1-returns"},{"type":"function","title":"Examples - LLMAgent.Store.new/1","doc":"iex> store = LLMAgent.Store.new()\n    iex> is_list(store.history) and is_list(store.thoughts) and is_list(store.tool_calls)\n    true\n    \n    iex> store = LLMAgent.Store.new(%{user_id: \"123\"})\n    iex> store.user_id\n    \"123\"","ref":"LLMAgent.Store.html#new/1-examples"},{"type":"function","title":"LLMAgent.Store.optimize/2","doc":"Optimizes the state by trimming history and pruning thoughts.\n\nThis is a utility function that combines trim_history and prune_thoughts\nto optimize the entire state at once.","ref":"LLMAgent.Store.html#optimize/2"},{"type":"function","title":"Parameters - LLMAgent.Store.optimize/2","doc":"- `state` - The current store state\n- `options` - Options including :max_history and :max_thoughts","ref":"LLMAgent.Store.html#optimize/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.optimize/2","doc":"An optimized state.","ref":"LLMAgent.Store.html#optimize/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.optimize/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> # Add lots of history and thoughts\n    iex> optimized_state = LLMAgent.Store.optimize(state, max_history: 30, max_thoughts: 10)\n    iex> is_map(optimized_state)\n    true","ref":"LLMAgent.Store.html#optimize/2-examples"},{"type":"function","title":"LLMAgent.Store.prune_thoughts/2","doc":"Prunes the thoughts list to a maximum size to prevent memory bloat.","ref":"LLMAgent.Store.html#prune_thoughts/2"},{"type":"function","title":"Parameters - LLMAgent.Store.prune_thoughts/2","doc":"- `state` - The current store state\n- `max_thoughts` - The maximum number of thoughts to keep (default: 20)","ref":"LLMAgent.Store.html#prune_thoughts/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.prune_thoughts/2","doc":"An updated state with pruned thoughts.","ref":"LLMAgent.Store.html#prune_thoughts/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.prune_thoughts/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = Enum.reduce(1..30, state, fn n, acc -> LLMAgent.Store.add_thought(acc, \"Thought \" <> Integer.to_string(n)) end)\n    iex> pruned_state = LLMAgent.Store.prune_thoughts(state)\n    iex> length(pruned_state.thoughts) <= 20\n    true","ref":"LLMAgent.Store.html#prune_thoughts/2-examples"},{"type":"function","title":"LLMAgent.Store.set_preferences/2","doc":"Sets user preferences in the state.","ref":"LLMAgent.Store.html#set_preferences/2"},{"type":"function","title":"Parameters - LLMAgent.Store.set_preferences/2","doc":"- `state` - The current store state\n- `preferences` - The preferences to set","ref":"LLMAgent.Store.html#set_preferences/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.set_preferences/2","doc":"An updated state with the preferences merged into the existing preferences.","ref":"LLMAgent.Store.html#set_preferences/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.set_preferences/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.set_preferences(state, %{language: \"en\"})\n    iex> state.preferences.language\n    \"en\"","ref":"LLMAgent.Store.html#set_preferences/2-examples"},{"type":"function","title":"LLMAgent.Store.trim_history/2","doc":"Trims history to a maximum number of entries while preserving system messages.\n\nThis function is useful for managing memory usage in long conversations.\nSystem messages are always preserved, and the most recent user/assistant \nmessages are kept up to the specified limit.","ref":"LLMAgent.Store.html#trim_history/2"},{"type":"function","title":"Parameters - LLMAgent.Store.trim_history/2","doc":"- `state` - The current store state\n- `max_entries` - The maximum number of entries to keep (default: 50)","ref":"LLMAgent.Store.html#trim_history/2-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.trim_history/2","doc":"An updated state with trimmed history.","ref":"LLMAgent.Store.html#trim_history/2-returns"},{"type":"function","title":"Examples - LLMAgent.Store.trim_history/2","doc":"iex> state = LLMAgent.Store.new()\n    iex> state = LLMAgent.Store.add_message(state, \"system\", \"You are an assistant\")\n    iex> # Add many user/assistant message pairs to exceed the limit\n    iex> state = Enum.reduce(1..60, state, fn n, acc -> \n    ...>   acc = LLMAgent.Store.add_message(acc, \"user\", \"Message \" <> Integer.to_string(n))\n    ...>   LLMAgent.Store.add_message(acc, \"assistant\", \"Response \" <> Integer.to_string(n))\n    ...> end)\n    iex> trimmed_state = LLMAgent.Store.trim_history(state)\n    iex> length(trimmed_state.history) <= 51  # 50 + 1 system message\n    true\n    iex> # Verify system message is preserved\n    iex> Enum.any?(trimmed_state.history, fn msg -> msg.role == \"system\" end)\n    true","ref":"LLMAgent.Store.html#trim_history/2-examples"},{"type":"function","title":"LLMAgent.Store.update_task_state/3","doc":"Updates task state for a specific task.","ref":"LLMAgent.Store.html#update_task_state/3"},{"type":"function","title":"Parameters - LLMAgent.Store.update_task_state/3","doc":"- `state` - The current store state\n- `task_id` - The ID of the task to update\n- `new_state` - The new state for the task","ref":"LLMAgent.Store.html#update_task_state/3-parameters"},{"type":"function","title":"Returns - LLMAgent.Store.update_task_state/3","doc":"An updated state with the task state updated.","ref":"LLMAgent.Store.html#update_task_state/3-returns"},{"type":"function","title":"Examples - LLMAgent.Store.update_task_state/3","doc":"iex> state = LLMAgent.Store.new()\n    iex> task = %{id: \"task_123\", status: \"running\"}\n    iex> state = LLMAgent.Store.add_task(state, task)\n    iex> state = LLMAgent.Store.update_task_state(state, \"task_123\", \"completed\")\n    iex> [%{id: \"task_123\", status: \"completed\"}] = state.current_tasks","ref":"LLMAgent.Store.html#update_task_state/3-examples"},{"type":"module","title":"LLMAgent.Providers.Anthropic","doc":"Anthropic provider implementation for LLMAgent.\n\nThis module implements the LLM provider interface for Anthropic, handling\nAPI calls, response parsing, and error handling specific to Anthropic's API.","ref":"LLMAgent.Providers.Anthropic.html"},{"type":"function","title":"LLMAgent.Providers.Anthropic.completion/1","doc":"Makes an Anthropic chat completion API call.","ref":"LLMAgent.Providers.Anthropic.html#completion/1"},{"type":"function","title":"Parameters - LLMAgent.Providers.Anthropic.completion/1","doc":"- `params` - A map containing the request parameters","ref":"LLMAgent.Providers.Anthropic.html#completion/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Providers.Anthropic.completion/1","doc":"- `{:ok, response}` - On success, returns the parsed response\n- `{:error, reason}` - On failure, returns the error reason","ref":"LLMAgent.Providers.Anthropic.html#completion/1-returns"},{"type":"function","title":"LLMAgent.Providers.Anthropic.embedding/1","doc":"Generates embeddings for the provided text using a compatible model.\n\nNote: Anthropic doesn't provide a dedicated embedding API, so this implementation\nuses a third-party compatible service or delegates to OpenAI's embedding API.","ref":"LLMAgent.Providers.Anthropic.html#embedding/1"},{"type":"function","title":"Parameters - LLMAgent.Providers.Anthropic.embedding/1","doc":"- `params` - A map with parameters for the request:\n  - `input` - The text to generate embeddings for\n  - `provider` - The embedding provider to use (default: :openai)","ref":"LLMAgent.Providers.Anthropic.html#embedding/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Providers.Anthropic.embedding/1","doc":"`{:ok, embeddings}` on success, `{:error, reason}` on failure.","ref":"LLMAgent.Providers.Anthropic.html#embedding/1-returns"},{"type":"function","title":"Examples - LLMAgent.Providers.Anthropic.embedding/1","doc":"iex> params = %{\n    ...>   input: \"Hello, world!\",\n    ...>   provider: :openai\n    ...> }\n    iex> {:ok, embeddings} = LLMAgent.Providers.Anthropic.embedding(params)\n    iex> is_list(embeddings)\n    true","ref":"LLMAgent.Providers.Anthropic.html#embedding/1-examples"},{"type":"module","title":"LLMAgent.Providers.OpenAI","doc":"OpenAI provider implementation for LLMAgent.\n\nThis module implements the LLM provider interface for OpenAI, handling\nAPI calls, response parsing, and error handling specific to OpenAI's API.","ref":"LLMAgent.Providers.OpenAI.html"},{"type":"function","title":"LLMAgent.Providers.OpenAI.completion/1","doc":"Sends a completion request to OpenAI's API.","ref":"LLMAgent.Providers.OpenAI.html#completion/1"},{"type":"function","title":"Parameters - LLMAgent.Providers.OpenAI.completion/1","doc":"- `params` - A map with parameters for the request:\n  - `model` - The model to use (e.g., \"gpt-4\")\n  - `messages` - The conversation history\n  - `tools` - Available tools for function calling\n  - `temperature` - Controls randomness (0.0 to 2.0)\n  - `max_tokens` - Maximum tokens to generate","ref":"LLMAgent.Providers.OpenAI.html#completion/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Providers.OpenAI.completion/1","doc":"`{:ok, response}` on success, `{:error, reason}` on failure.","ref":"LLMAgent.Providers.OpenAI.html#completion/1-returns"},{"type":"function","title":"Examples - LLMAgent.Providers.OpenAI.completion/1","doc":"iex> params = %{\n    ...>   model: \"gpt-4\",\n    ...>   messages: [%{role: \"user\", content: \"Hello\"}],\n    ...>   max_tokens: 500\n    ...> }\n    iex> {:ok, response} = LLMAgent.Providers.OpenAI.completion(params)\n    iex> is_map(response)\n    true","ref":"LLMAgent.Providers.OpenAI.html#completion/1-examples"},{"type":"function","title":"LLMAgent.Providers.OpenAI.embedding/1","doc":"Generates embeddings for the provided text using OpenAI's API.","ref":"LLMAgent.Providers.OpenAI.html#embedding/1"},{"type":"function","title":"Parameters - LLMAgent.Providers.OpenAI.embedding/1","doc":"- `params` - A map with parameters for the request:\n  - `model` - The embedding model to use (e.g., \"text-embedding-ada-002\")\n  - `input` - The text to generate embeddings for","ref":"LLMAgent.Providers.OpenAI.html#embedding/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Providers.OpenAI.embedding/1","doc":"`{:ok, embeddings}` on success, `{:error, reason}` on failure.","ref":"LLMAgent.Providers.OpenAI.html#embedding/1-returns"},{"type":"function","title":"Examples - LLMAgent.Providers.OpenAI.embedding/1","doc":"iex> params = %{\n    ...>   model: \"text-embedding-ada-002\",\n    ...>   input: \"Hello, world!\"\n    ...> }\n    iex> {:ok, embeddings} = LLMAgent.Providers.OpenAI.embedding(params)\n    iex> is_list(embeddings)\n    true","ref":"LLMAgent.Providers.OpenAI.html#embedding/1-examples"},{"type":"module","title":"LLMAgent.Plugin","doc":"Implements the AgentForge.Plugin behavior for LLM integrations.\n\nThis module provides a plugin implementation that registers LLM-specific tools\nand allows LLMAgent to integrate with different LLM providers.","ref":"LLMAgent.Plugin.html"},{"type":"function","title":"LLMAgent.Plugin.call_llm/1","doc":"Calls an LLM service with the provided parameters.","ref":"LLMAgent.Plugin.html#call_llm/1"},{"type":"function","title":"Parameters - LLMAgent.Plugin.call_llm/1","doc":"- `params` - A map with parameters for the LLM call:\n  - `provider` - The LLM provider (e.g., :openai, :anthropic)\n  - `messages` - The conversation history\n  - `tools` - Available tools\n  - `options` - Additional provider-specific options","ref":"LLMAgent.Plugin.html#call_llm/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Plugin.call_llm/1","doc":"A map with the LLM response.","ref":"LLMAgent.Plugin.html#call_llm/1-returns"},{"type":"function","title":"LLMAgent.Plugin.init/1","doc":"Initializes the LLMAgent plugin.\n\nChecks that required dependencies are available for the specified provider.","ref":"LLMAgent.Plugin.html#init/1"},{"type":"function","title":"Parameters - LLMAgent.Plugin.init/1","doc":"- `opts` - Plugin options including provider selection","ref":"LLMAgent.Plugin.html#init/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Plugin.init/1","doc":"`:ok` if initialization succeeds, `{:error, reason}` otherwise.","ref":"LLMAgent.Plugin.html#init/1-returns"},{"type":"function","title":"Examples - LLMAgent.Plugin.init/1","doc":"iex> LLMAgent.Plugin.init(provider: :mock)\n    :ok","ref":"LLMAgent.Plugin.html#init/1-examples"},{"type":"function","title":"LLMAgent.Plugin.metadata/0","doc":"Returns metadata about the LLMAgent plugin.","ref":"LLMAgent.Plugin.html#metadata/0"},{"type":"function","title":"Returns - LLMAgent.Plugin.metadata/0","doc":"A map with plugin metadata.","ref":"LLMAgent.Plugin.html#metadata/0-returns"},{"type":"function","title":"LLMAgent.Plugin.parse_llm_response/1","doc":"Parses an LLM response into a structured format.","ref":"LLMAgent.Plugin.html#parse_llm_response/1"},{"type":"function","title":"Parameters - LLMAgent.Plugin.parse_llm_response/1","doc":"- `params` - A map with parameters:\n  - `response` - The raw LLM response\n  - `format` - The desired output format","ref":"LLMAgent.Plugin.html#parse_llm_response/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Plugin.parse_llm_response/1","doc":"A structured representation of the LLM response.","ref":"LLMAgent.Plugin.html#parse_llm_response/1-returns"},{"type":"function","title":"LLMAgent.Plugin.register_primitives/0","doc":"Registers LLM-specific primitives with AgentForge.","ref":"LLMAgent.Plugin.html#register_primitives/0"},{"type":"function","title":"Returns - LLMAgent.Plugin.register_primitives/0","doc":"`:ok` if registration succeeds.","ref":"LLMAgent.Plugin.html#register_primitives/0-returns"},{"type":"function","title":"LLMAgent.Plugin.register_tools/1","doc":"Registers LLM-specific tools with AgentForge.","ref":"LLMAgent.Plugin.html#register_tools/1"},{"type":"function","title":"Parameters - LLMAgent.Plugin.register_tools/1","doc":"- `registry` - The AgentForge tool registry","ref":"LLMAgent.Plugin.html#register_tools/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Plugin.register_tools/1","doc":"`:ok` if registration succeeds.","ref":"LLMAgent.Plugin.html#register_tools/1-returns"},{"type":"module","title":"LLMAgent.Tasks","doc":"Manages long-running tasks using AgentForge's execution capabilities.\n\nThis module provides functions for starting, monitoring, and controlling\ntasks that are implemented as sequences of AgentForge primitives.","ref":"LLMAgent.Tasks.html"},{"type":"function","title":"LLMAgent.Tasks.cancel/1","doc":"Attempts to cancel a running task.","ref":"LLMAgent.Tasks.html#cancel/1"},{"type":"function","title":"Parameters - LLMAgent.Tasks.cancel/1","doc":"- `_task_id` - The ID of the task to cancel","ref":"LLMAgent.Tasks.html#cancel/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Tasks.cancel/1","doc":"- `:ok` - If the task was successfully cancelled\n- `{:error, reason}` - If the task could not be cancelled","ref":"LLMAgent.Tasks.html#cancel/1-returns"},{"type":"function","title":"LLMAgent.Tasks.get_stats/1","doc":"Gets the latest statistics for a task.","ref":"LLMAgent.Tasks.html#get_stats/1"},{"type":"function","title":"Parameters - LLMAgent.Tasks.get_stats/1","doc":"- `_task_id` - The ID of the task","ref":"LLMAgent.Tasks.html#get_stats/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Tasks.get_stats/1","doc":"A map of task statistics.","ref":"LLMAgent.Tasks.html#get_stats/1-returns"},{"type":"function","title":"LLMAgent.Tasks.pause/1","doc":"Attempts to pause a running task.","ref":"LLMAgent.Tasks.html#pause/1"},{"type":"function","title":"Parameters - LLMAgent.Tasks.pause/1","doc":"- `_task_id` - The ID of the task to pause","ref":"LLMAgent.Tasks.html#pause/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Tasks.pause/1","doc":"- `:ok` - If the task was successfully paused\n- `{:error, reason}` - If the task could not be paused","ref":"LLMAgent.Tasks.html#pause/1-returns"},{"type":"function","title":"LLMAgent.Tasks.resume/1","doc":"Attempts to resume a paused task.","ref":"LLMAgent.Tasks.html#resume/1"},{"type":"function","title":"Parameters - LLMAgent.Tasks.resume/1","doc":"- `_task_id` - The ID of the task to resume","ref":"LLMAgent.Tasks.html#resume/1-parameters"},{"type":"function","title":"Returns - LLMAgent.Tasks.resume/1","doc":"- `:ok` - If the task was successfully resumed\n- `{:error, reason}` - If the task could not be resumed","ref":"LLMAgent.Tasks.html#resume/1-returns"},{"type":"function","title":"LLMAgent.Tasks.start/4","doc":"Starts a task with the given definition and parameters.","ref":"LLMAgent.Tasks.html#start/4"},{"type":"function","title":"Parameters - LLMAgent.Tasks.start/4","doc":"- `task_def` - The task definition as a list of AgentForge primitives\n- `params` - Parameters for the task\n- `state` - The current agent state\n- `opts` - Additional options for task execution","ref":"LLMAgent.Tasks.html#start/4-parameters"},{"type":"function","title":"Returns - LLMAgent.Tasks.start/4","doc":"A tuple containing the task ID and a task state signal.","ref":"LLMAgent.Tasks.html#start/4-returns"},{"type":"function","title":"Examples - LLMAgent.Tasks.start/4","doc":"iex> task_def = [\n    ...>   AgentForge.Primitives.transform(fn data -> Map.put(data, :processed, true) end)\n    ...> ]\n    iex> params = %{data: \"test\"}\n    iex> state = %{}\n    iex> {task_id, signal} = LLMAgent.Tasks.start(task_def, params, state)\n    iex> is_binary(task_id) and signal.type == :task_state\n    true","ref":"LLMAgent.Tasks.html#start/4-examples"},{"type":"extras","title":"LLMAgent","doc":"# LLMAgent\n\n[![CI](https://github.com/i365dev/llm_agent/actions/workflows/ci.yml/badge.svg)](https://github.com/i365dev/llm_agent/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/llm_agent.svg)](https://hex.pm/packages/llm_agent)\n[![Docs](https://img.shields.io/badge/hex-docs-blue.svg)](https://hexdocs.pm/llm_agent)\n[![License](https://img.shields.io/badge/license-MIT-blue)](https://github.com/i365dev/llm_agent/blob/main/LICENSE)\n\nLLMAgent is an abstraction library for building domain-specific intelligent agents based on Large Language Models (LLMs). Built on top of AgentForge's signal-driven architecture, LLMAgent provides specialized patterns for LLM-powered agents, including predefined signals, handlers, store structures, and flows optimized for conversational agents.","ref":"readme.html"},{"type":"extras","title":"Features - LLMAgent","doc":"- 🧠 LLM-specific interaction patterns and signal types\n- 🔀 Message processing workflows and handlers\n- 🛠️ Tool integration and execution\n- ⏱️ Long-running task management\n- 💬 Context and conversation management\n- 🔌 Plugin-based provider integrations\n- 🔄 AgentForge compatibility","ref":"readme.html#features"},{"type":"extras","title":"Installation - LLMAgent","doc":"Add `llm_agent` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:llm_agent, \"~> 0.0.1\"},\n    # Optional LLM provider dependencies\n    {:openai, \"~> 0.5.0\"}, # If using OpenAI\n    {:anthropic, \"~> 0.1.0\"} # If using Anthropic\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - LLMAgent","doc":"```elixir\n# Create agent with system prompt and basic tools\n{flow, initial_state} = LLMAgent.new(\n  \"You are a helpful assistant that can answer questions and use tools.\",\n  [\n    %{\n      name: \"search\",\n      description: \"Search the web for information\",\n      parameters: %{\n        \"type\" => \"object\",\n        \"properties\" => %{\n          \"query\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"Search query\"\n          }\n        },\n        \"required\" => [\"query\"]\n      },\n      execute: &MyApp.Tools.search/1\n    }\n  ]\n)\n\n# Process a user message\n{:ok, result, new_state} = LLMAgent.process(flow, initial_state, \"What is the capital of France?\")\n\n# Handle result\ncase result do\n  %{type: :response, data: content} ->\n    IO.puts(\"Agent response: #{content}\")\n    \n  %{type: :thinking, data: thought} ->\n    IO.puts(\"Agent thinking: #{thought}\")\n    \n  %{type: :error, data: %{message: message}} ->\n    IO.puts(\"Error: #{message}\")\nend\n```","ref":"readme.html#quick-start"},{"type":"extras","title":"Core Components - LLMAgent","doc":"","ref":"readme.html#core-components"},{"type":"extras","title":"Signals - LLMAgent","doc":"LLMAgent defines specialized signal types for LLM interactions:\n\n```elixir\n# Create a user message signal\nsignal = LLMAgent.Signals.user_message(\"Help me analyze AAPL stock\")\n\n# Create a thinking signal\nthinking = LLMAgent.Signals.thinking(\"I need to get stock price data\", 1)\n\n# Create a tool call signal\ntool_call = LLMAgent.Signals.tool_call(\"get_stock_price\", %{ticker: \"AAPL\"})\n```","ref":"readme.html#signals"},{"type":"extras","title":"Handlers - LLMAgent","doc":"Handlers process LLM-specific signals:\n\n```elixir\n# Message handler processes user messages\nLLMAgent.Handlers.message_handler(signal, state)\n\n# Tool handler executes tool calls\nLLMAgent.Handlers.tool_handler(tool_call, state)\n```","ref":"readme.html#handlers"},{"type":"extras","title":"Store - LLMAgent","doc":"Manage conversation state:\n\n```elixir\n# Create a new store\nstate = LLMAgent.Store.new()\n\n# Add a message to history\nstate = LLMAgent.Store.add_message(state, \"user\", \"Hello\")\n\n# Get conversation history\nhistory = LLMAgent.Store.get_llm_history(state)\n```","ref":"readme.html#store"},{"type":"extras","title":"Flows - LLMAgent","doc":"Create standard workflow compositions:\n\n```elixir\n# Create a conversation flow with tools\n{flow, state} = LLMAgent.Flows.conversation(system_prompt, tools)\n\n# Create a simple QA agent\n{flow, state} = LLMAgent.Flows.qa_agent(system_prompt)\n```","ref":"readme.html#flows"},{"type":"extras","title":"Tasks - LLMAgent","doc":"Manage long-running operations:\n\n```elixir\n# Define a task with AgentForge primitives\ntask_def = [\n  AgentForge.Primitives.transform(fn data -> Map.put(data, :processed, true) end)\n]\n\n# Start the task\n{task_id, signal} = LLMAgent.Tasks.start(task_def, params, state)\n```","ref":"readme.html#tasks"},{"type":"extras","title":"LLM Providers - LLMAgent","doc":"LLMAgent supports multiple LLM providers through its plugin system:\n\n```elixir\n# Use OpenAI for completions\n{:ok, response} = LLMAgent.Providers.OpenAI.completion(%{\n  model: \"gpt-4\",\n  messages: [%{role: \"user\", content: \"Hello\"}],\n  max_tokens: 500\n})\n\n# Use Anthropic for completions\n{:ok, response} = LLMAgent.Providers.Anthropic.completion(%{\n  model: \"claude-3-opus-20240229\",\n  messages: [%{role: \"user\", content: \"Hello\"}],\n  max_tokens: 500\n})\n```","ref":"readme.html#llm-providers"},{"type":"extras","title":"Domain-Specific Agents - LLMAgent","doc":"Build specialized agents by:\n\n1. Creating domain-specific handlers\n2. Registering domain-specific tools\n3. Defining domain-specific tasks\n4. Creating domain-specific flows\n\n```elixir\ndefmodule MyApp.InvestmentAgent do\n  def new(options \\\\ %{}) do\n    # Define system prompt\n    system_prompt = \"You are an AI investment assistant...\"\n    \n    # Define investment tools\n    tools = [\n      %{\n        name: \"get_stock_price\",\n        description: \"Get current price for a stock\",\n        parameters: %{\n          \"type\" => \"object\",\n          \"properties\" => %{\n            \"ticker\" => %{\n              \"type\" => \"string\",\n              \"description\" => \"Stock ticker symbol\"\n            }\n          },\n          \"required\" => [\"ticker\"]\n        },\n        execute: &MyApp.Tools.get_stock_price/1\n      }\n    ]\n    \n    # Create agent flow\n    LLMAgent.new(system_prompt, tools, options)\n  end\nend\n```","ref":"readme.html#domain-specific-agents"},{"type":"extras","title":"Additional Documentation - LLMAgent","doc":"- [Getting Started Guide](https://hexdocs.pm/llm_agent/getting_started.html)\n- [Architecture Guide](https://hexdocs.pm/llm_agent/architecture.html)\n- [Tool Integration](https://hexdocs.pm/llm_agent/tool_integration.html)\n- [Custom Agents](https://hexdocs.pm/llm_agent/custom_agents.html)\n- [API Reference](https://hexdocs.pm/llm_agent/api-reference.html)","ref":"readme.html#additional-documentation"},{"type":"extras","title":"Contributing - LLMAgent","doc":"We welcome contributions from the community! Please see our [Contributing Guide](CONTRIBUTING.md) for more information on how to get involved.","ref":"readme.html#contributing"},{"type":"extras","title":"License - LLMAgent","doc":"LLMAgent is released under the MIT License. See the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to LLMAgent will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Comprehensive documentation structure with guides\n- Examples demonstrating various agent patterns\n- CI/CD workflows for automated testing and releases\n- Enhanced mix.exs configuration for better docs","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Compilation errors in AgentForge flow functions\n- Undefined variable 'i' in doctest examples\n- Refactored task_flow and batch_processing functions\n\n## [0.0.1] - 2025-03-27","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of redesigned LLMAgent library\n- Signal-based architecture built on AgentForge\n- Tool integration system\n- Handlers for various signal types\n- Store for managing conversation state\n- Flow compositions for different agent patterns","ref":"changelog.html#added"},{"type":"extras","title":"Contributing to LLMAgent","doc":"# Contributing to LLMAgent\n\nThank you for your interest in contributing to LLMAgent! This document provides guidelines and instructions for contributing to this project.","ref":"contributing.html"},{"type":"extras","title":"Code of Conduct - Contributing to LLMAgent","doc":"Please read and follow our [Code of Conduct](CODE_OF_CONDUCT.md). We expect all contributors to adhere to these guidelines to ensure a welcoming and productive environment.","ref":"contributing.html#code-of-conduct"},{"type":"extras","title":"Getting Started - Contributing to LLMAgent","doc":"1. **Fork the repository** on GitHub.\n2. **Clone your fork** locally:\n   ```bash\n   git clone https://github.com/YOUR-USERNAME/llm_agent.git\n   cd llm_agent\n   ```\n3. **Install dependencies**:\n   ```bash\n   mix deps.get\n   ```\n4. **Create a branch** for your changes:\n   ```bash\n   git checkout -b feature/your-feature-name\n   ```","ref":"contributing.html#getting-started"},{"type":"extras","title":"Development Workflow - Contributing to LLMAgent","doc":"","ref":"contributing.html#development-workflow"},{"type":"extras","title":"Code Style & Standards - Contributing to LLMAgent","doc":"LLMAgent follows the standard Elixir style guidelines:\n\n1. Use `mix format` to format your code before submitting\n2. Follow the naming conventions:\n   - **Modules**: CamelCase, descriptive of purpose\n   - **Functions**: snake_case, verb-based, descriptive\n   - **Variables**: snake_case, clear purpose\n   - **Signals**: Atom types, descriptive of state\n   - **Constants**: ALL_CAPS for true constants, CamelCase for configuration\n\n3. Write clear and expressive code, preferring readability over cleverness","ref":"contributing.html#code-style-standards"},{"type":"extras","title":"Documentation - Contributing to LLMAgent","doc":"All code contributions should be properly documented:\n\n1. **Module Documentation**:\n   - Every module must have a `@moduledoc`\n   - Explain the module's purpose and responsibilities\n   - Document how it fits in the overall architecture\n\n2. **Function Documentation**:\n   - All public functions must have a `@doc` comment\n   - Include `@spec` type specifications\n   - Document parameters and return values\n   - Provide examples for non-trivial functions","ref":"contributing.html#documentation"},{"type":"extras","title":"Testing - Contributing to LLMAgent","doc":"1. All new features or bug fixes should include tests:\n   ```bash\n   mix test\n   ```\n\n2. Ensure your changes don't break existing functionality.\n\n3. Test coverage should be maintained or improved:\n   ```bash\n   mix test --cover\n   ```\n\n4. For LLM-related components, include appropriate mocks to avoid API calls during tests.","ref":"contributing.html#testing"},{"type":"extras","title":"Linting & Static Analysis - Contributing to LLMAgent","doc":"Before submitting changes, run:\n\n```bash\nmix lint  # Alias for format and credo\n```\n\nThis checks code formatting and runs Credo for static analysis.","ref":"contributing.html#linting-static-analysis"},{"type":"extras","title":"Submitting Changes - Contributing to LLMAgent","doc":"1. **Commit your changes** with clear, descriptive commit messages:\n   ```bash\n   git commit -m \"Add feature X that solves problem Y\"\n   ```\n\n2. **Push to your fork**:\n   ```bash\n   git push origin feature/your-feature-name\n   ```\n\n3. **Create a Pull Request** against the `develop` branch of the original repository.\n\n4. **Describe your changes** in the PR description, including:\n   - What problem your PR solves\n   - How your implementation works\n   - Any potential side effects or considerations\n   - Links to related issues\n\n5. **Wait for review**. Maintainers will review your PR and might request changes.","ref":"contributing.html#submitting-changes"},{"type":"extras","title":"Types of Contributions - Contributing to LLMAgent","doc":"","ref":"contributing.html#types-of-contributions"},{"type":"extras","title":"Bug Fixes - Contributing to LLMAgent","doc":"If you're fixing a bug:\n\n1. Ensure the bug is documented in an issue\n2. Reference the issue in your PR\n3. Add a test that reproduces the bug\n4. Explain your approach to fixing it","ref":"contributing.html#bug-fixes"},{"type":"extras","title":"Features - Contributing to LLMAgent","doc":"For new features:\n\n1. Open an issue discussing the feature before implementing\n2. Explain how the feature aligns with project goals\n3. Consider edge cases and performance implications\n4. Include comprehensive tests and documentation","ref":"contributing.html#features"},{"type":"extras","title":"Documentation - Contributing to LLMAgent","doc":"Documentation improvements are always welcome:\n\n1. Correct inaccuracies in existing docs\n2. Expand explanations for clarity\n3. Add examples where useful\n4. Ensure formatting is consistent","ref":"contributing.html#documentation"},{"type":"extras","title":"Architectural Guidelines - Contributing to LLMAgent","doc":"When contributing to LLMAgent, follow these architectural principles:\n\n1. **Signal-Driven Architecture**: Maintain the separation between signals, handlers, and state.\n2. **Immutability**: State should be immutable and transformations explicit.\n3. **Composability**: Components should be composable and reusable.\n4. **Separation of Concerns**: Clearly delineate responsibilities between modules.\n5. **Testability**: Design for testability, avoid hidden dependencies.","ref":"contributing.html#architectural-guidelines"},{"type":"extras","title":"Release Process - Contributing to LLMAgent","doc":"Only maintainers can release new versions. The process involves:\n\n1. Updating the CHANGELOG.md\n2. Bumping version in mix.exs\n3. Tagging the release\n4. Publishing to Hex.pm","ref":"contributing.html#release-process"},{"type":"extras","title":"Getting Help - Contributing to LLMAgent","doc":"If you need help with the contribution process:\n\n1. Check existing documentation\n2. Open a discussion on GitHub\n3. Reach out to maintainers\n\nThank you for contributing to LLMAgent!","ref":"contributing.html#getting-help"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 i365 Tech\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"LLMAgent Architecture","doc":"# LLMAgent Architecture\n\nThis guide provides an overview of the LLMAgent architecture, explaining how it builds on AgentForge's signal-driven design while providing LLM-specific abstractions.","ref":"architecture.html"},{"type":"extras","title":"Core Principles - LLMAgent Architecture","doc":"LLMAgent is designed with the following principles in mind:\n\n1. **LLM-Specific Abstractions**: Create patterns optimized for LLM interactions\n2. **Separation of Concerns**: Clearly delineate LLM logic from infrastructure\n3. **Elixir Ecosystem Integration**: Leverage the strengths of Elixir/OTP\n4. **Lightweight Implementation**: Maintain a clean, minimal codebase\n5. **Testability**: Ensure components can be tested in isolation","ref":"architecture.html#core-principles"},{"type":"extras","title":"System Architecture - LLMAgent Architecture","doc":"LLMAgent extends AgentForge's signal-driven architecture with components specifically designed for LLM interactions:\n\n```mermaid\ngraph TD\n    User[User Input] --> Signals\n    Signals --> Flow\n    Flow --> Handlers\n    Handlers --> Store\n    Store --> LLM[LLM Provider]\n    LLM --> Store\n    Store --> Tools\n    Tools --> Store\n    Store --> Response[Response]\n```","ref":"architecture.html#system-architecture"},{"type":"extras","title":"Core Components - LLMAgent Architecture","doc":"1. **Signals**: Represent events in the agent lifecycle\n2. **Handlers**: Process signals and update state\n3. **Store**: Manages conversation state and history\n4. **Flows**: Combine handlers into coherent sequences\n5. **Providers**: Interface with LLM backends\n6. **Tools**: External capabilities the agent can use","ref":"architecture.html#core-components"},{"type":"extras","title":"Signal Flow - LLMAgent Architecture","doc":"The typical flow of a conversation follows these steps:\n\n1. User input is converted to a `:user_message` signal\n2. The message handler processes the user message and generates a `:thinking` signal\n3. The thinking handler calls the LLM and decides whether to use a tool or generate a response\n4. If using a tool, it generates a `:tool_call` signal\n5. The tool handler executes the tool and generates a `:tool_result` signal\n6. The tool result handler incorporates the result and generates a new `:thinking` signal\n7. Eventually, the LLM generates a response, creating a `:response` signal\n8. The response handler formats and returns the final response","ref":"architecture.html#signal-flow"},{"type":"extras","title":"Component Diagram - LLMAgent Architecture","doc":"Here is a detailed component diagram showing the relationships between the main modules:\n\n```mermaid\nclassDiagram\n    class LLMAgent.Signals {\n        +user_message(content)\n        +thinking(content)\n        +tool_call(name, args)\n        +tool_result(name, result)\n        +response(content)\n        +error(message, source)\n    }\n    \n    class LLMAgent.Handlers {\n        +message_handler(signal, state)\n        +thinking_handler(signal, state)\n        +tool_handler(signal, state)\n        +tool_result_handler(signal, state)\n        +response_handler(signal, state)\n        +error_handler(signal, state)\n    }\n    \n    class LLMAgent.Store {\n        +new(initial_state)\n        +add_message(state, role, content)\n        +add_thought(state, content)\n        +add_tool_call(state, name, args, result)\n        +prune_thoughts(state, max_thoughts)\n    }\n    \n    class LLMAgent.Flows {\n        +conversation(system_prompt, tools, options)\n        +task_flow(task_definition, options)\n        +batch_processing(items, batch_handler, options)\n        +qa_agent(system_prompt, options)\n    }\n    \n    LLMAgent.Signals -- LLMAgent.Handlers\n    LLMAgent.Handlers -- LLMAgent.Store\n    LLMAgent.Flows -- LLMAgent.Handlers\n    LLMAgent.Flows -- LLMAgent.Store\n```","ref":"architecture.html#component-diagram"},{"type":"extras","title":"Extension Points - LLMAgent Architecture","doc":"LLMAgent is designed to be extended in several ways:\n\n1. **Custom Handlers**: Create specialized handlers for domain-specific signals\n2. **Custom Flows**: Combine handlers in new ways for different interaction patterns\n3. **Custom Tools**: Add new capabilities to your agent\n4. **Provider Plugins**: Integrate with different LLM backends","ref":"architecture.html#extension-points"},{"type":"extras","title":"Design Decisions - LLMAgent Architecture","doc":"","ref":"architecture.html#design-decisions"},{"type":"extras","title":"Why Signal-Driven Architecture? - LLMAgent Architecture","doc":"The signal-driven architecture provides several benefits:\n\n1. **Composability**: Handlers can be combined in flexible ways\n2. **Testability**: Each component can be tested in isolation\n3. **Extensibility**: New signals and handlers can be added without changing existing code\n4. **Visibility**: The flow of information is explicit and traceable","ref":"architecture.html#why-signal-driven-architecture"},{"type":"extras","title":"Why Elixir? - LLMAgent Architecture","doc":"Elixir's functional nature, pattern matching, and supervision trees make it ideal for building reliable, maintainable agent systems:\n\n1. **Immutable State**: Ensures predictable state transitions\n2. **Pattern Matching**: Makes signal handling elegant and explicit\n3. **Concurrency**: Allows handling multiple conversations efficiently\n4. **Fault Tolerance**: Supervisors can restart failed components","ref":"architecture.html#why-elixir"},{"type":"extras","title":"Next Steps - LLMAgent Architecture","doc":"- Explore [tool integration](tool_integration.html)\n- Learn how to create [custom agents](custom_agents.html)\n- Understand [LLM provider integration](provider_guide.html)","ref":"architecture.html#next-steps"},{"type":"extras","title":"Creating Custom Agents","doc":"# Creating Custom Agents\n\nThis guide explains how to create domain-specific LLM agents by extending the base functionality of LLMAgent. Custom agents can be specialized for particular tasks, domains, or interaction patterns.","ref":"custom_agents.html"},{"type":"extras","title":"Domain-Specific Agents - Creating Custom Agents","doc":"While the standard conversation flow works well for general-purpose assistants, domain-specific agents can provide more specialized capabilities:\n\n- **Financial advisors** that understand investment concepts\n- **Technical support agents** with product knowledge\n- **Medical assistants** that can interpret symptoms\n- **Educational tutors** with teaching methodologies","ref":"custom_agents.html#domain-specific-agents"},{"type":"extras","title":"Extending Base Flows - Creating Custom Agents","doc":"To create a custom agent, start by extending one of the base flows:\n\n```elixir\ndefmodule MyApp.FinancialAdvisor do\n  alias LLMAgent.{Flows, Signals, Store}\n  \n  def create_agent(options \\\\ []) do\n    # Define a domain-specific system prompt\n    system_prompt = \"\"\"\n    You are a financial advisor specializing in retirement planning.\n    Follow these guidelines when advising clients:\n    - Always ask about their time horizon and risk tolerance\n    - Recommend diversified investment strategies\n    - Explain concepts in clear, non-technical language\n    - Disclose that you're not providing personalized financial advice\n    \"\"\"\n    \n    # Define domain-specific tools\n    tools = [\n      %{\n        name: \"calculate_compound_interest\",\n        description: \"Calculate compound interest growth. Parameters: principal, rate, time, compounds_per_year\",\n        execute: &MyApp.FinancialTools.calculate_compound_interest/1\n      },\n      %{\n        name: \"retirement_calculator\",\n        description: \"Estimate retirement savings needed. Parameters: current_age, retirement_age, life_expectancy, annual_expenses\",\n        execute: &MyApp.FinancialTools.retirement_calculator/1\n      }\n    ]\n    \n    # Create the base conversation flow\n    {flow, initial_state} = Flows.conversation(system_prompt, tools, options)\n    \n    # Add custom middleware if needed\n    custom_flow = fn signal, state ->\n      # Log all interactions for compliance\n      log_interaction(signal, state)\n      \n      # Apply the base flow\n      flow.(signal, state)\n    end\n    \n    {custom_flow, initial_state}\n  end\n  \n  defp log_interaction(signal, state) do\n    # Implement compliance logging\n    # ...\n  end\nend\n```","ref":"custom_agents.html#extending-base-flows"},{"type":"extras","title":"Custom Handlers - Creating Custom Agents","doc":"You can create custom handlers for specialized signal processing:\n\n```elixir\ndefmodule MyApp.MedicalHandlers do\n  alias LLMAgent.{Signals, Store}\n  \n  def symptom_handler(%{type: :symptom, data: symptoms}, state) do\n    # Process symptoms and generate potential diagnoses\n    # ...\n    \n    # Create a thinking signal with medical context\n    thinking = Signals.thinking(\"Analyzing symptoms: #{symptoms}\")\n    {{:emit, thinking}, state}\n  end\n  \n  def diagnosis_handler(%{type: :diagnosis, data: diagnosis}, state) do\n    # Format and validate the diagnosis\n    # ...\n    \n    # Return a response with recommendations\n    response = Signals.response(formatted_diagnosis)\n    {{:emit, response}, state}\n  end\nend\n```","ref":"custom_agents.html#custom-handlers"},{"type":"extras","title":"Custom Signal Types - Creating Custom Agents","doc":"Define domain-specific signal types to represent specialized concepts:\n\n```elixir\ndefmodule MyApp.MedicalSignals do\n  alias AgentForge.Signal\n  \n  def symptom(description) do\n    Signal.new(:symptom, description)\n  end\n  \n  def diagnosis(condition, confidence, recommendations) do\n    Signal.new(:diagnosis, %{\n      condition: condition,\n      confidence: confidence,\n      recommendations: recommendations\n    })\n  end\n  \n  def prescription(medication, dosage, instructions) do\n    Signal.new(:prescription, %{\n      medication: medication,\n      dosage: dosage,\n      instructions: instructions\n    })\n  end\nend\n```","ref":"custom_agents.html#custom-signal-types"},{"type":"extras","title":"Custom State Management - Creating Custom Agents","doc":"Extend the store to include domain-specific state:\n\n```elixir\ndefmodule MyApp.MedicalStore do\n  alias LLMAgent.Store\n  \n  def new(initial_state \\\\ %{}) do\n    # Start with the base store\n    base_store = Store.new(initial_state)\n    \n    # Add medical-specific state\n    Map.merge(base_store, %{\n      patient_info: %{},\n      medical_history: [],\n      current_symptoms: [],\n      previous_diagnoses: []\n    })\n  end\n  \n  def add_patient_info(state, info) do\n    put_in(state, [:patient_info], info)\n  end\n  \n  def add_symptom(state, symptom) do\n    update_in(state, [:current_symptoms], &([symptom | &1]))\n  end\n  \n  def add_diagnosis(state, diagnosis) do\n    update_in(state, [:previous_diagnoses], &([diagnosis | &1]))\n  end\nend\n```","ref":"custom_agents.html#custom-state-management"},{"type":"extras","title":"Composing Custom Flows - Creating Custom Agents","doc":"Create domain-specific flow compositions for specialized interaction patterns:\n\n```elixir\ndefmodule MyApp.MedicalFlows do\n  alias LLMAgent.{Flows, Handlers}\n  alias MyApp.{MedicalHandlers, MedicalStore}\n  \n  def diagnosis_flow(system_prompt, tools \\\\ [], options \\\\ []) do\n    # Create base conversation flow\n    {base_flow, initial_state} = Flows.conversation(system_prompt, tools, options)\n    \n    # Initialize medical-specific state\n    medical_state = MedicalStore.new(initial_state)\n    \n    # Create a custom flow with specialized handlers\n    custom_flow = fn signal, state ->\n      state\n      |> handle_with(&Handlers.message_handler/2, signal)\n      |> handle_with(&MedicalHandlers.symptom_handler/2, signal)\n      |> handle_with(&Handlers.thinking_handler/2, signal)\n      |> handle_with(&Handlers.tool_handler/2, signal)\n      |> handle_with(&Handlers.tool_result_handler/2, signal)\n      |> handle_with(&MedicalHandlers.diagnosis_handler/2, signal)\n      |> handle_with(&Handlers.response_handler/2, signal)\n      |> handle_with(&Handlers.error_handler/2, signal)\n    end\n    \n    {custom_flow, medical_state}\n  end\n  \n  # Helper for handling signals, similar to LLMAgent.Flows implementation\n  defp handle_with({:halt, _} = result, _handler, _signal), do: result\n  defp handle_with({:skip, state}, handler, signal) do\n    # Implementation as in LLMAgent.Flows\n  end\n  # Other handle_with clauses...\nend\n```","ref":"custom_agents.html#composing-custom-flows"},{"type":"extras","title":"Best Practices - Creating Custom Agents","doc":"","ref":"custom_agents.html#best-practices"},{"type":"extras","title":"Domain-Specific System Prompts - Creating Custom Agents","doc":"Create detailed system prompts that include:\n\n1. **Role definition**: Clearly state the specialized role and expertise\n2. **Domain knowledge**: Include essential facts and concepts\n3. **Guidelines**: Provide rules and constraints for the domain\n4. **Examples**: Demonstrate ideal responses for common scenarios\n\nExample for a legal assistant:\n\n```elixir\nsystem_prompt = \"\"\"\nYou are a legal assistant specializing in contract law.\n\nEXPERTISE:\n- Contract formation and interpretation\n- Common contractual clauses and their implications\n- Basic legal principles (not legal advice)\n\nGUIDELINES:\n- Always clarify you are not providing legal advice\n- Recommend consulting with a licensed attorney for specific legal questions\n- Explain legal concepts in plain language\n- Ask clarifying questions when details are ambiguous\n\nEXAMPLE INTERACTIONS:\nUser: \"What should be included in an NDA?\"\nAssistant: \"A typical NDA (Non-Disclosure Agreement) generally includes: 1) Definition of confidential information, 2) Scope of confidentiality obligation, 3) Exclusions from confidential information, 4) Term of the agreement, and 5) Remedies for breach. However, please note that I'm not providing legal advice, and you should consult with an attorney to ensure your NDA meets your specific needs and complies with relevant laws.\"\n\"\"\"\n```","ref":"custom_agents.html#domain-specific-system-prompts"},{"type":"extras","title":"Configuration Options - Creating Custom Agents","doc":"Use options to make agents configurable:\n\n```elixir\ndef create_financial_advisor(options \\\\ []) do\n  # Extract customization options\n  risk_profile = Keyword.get(options, :risk_profile, :moderate)\n  timezone = Keyword.get(options, :timezone, \"UTC\")\n  include_market_data = Keyword.get(options, :include_market_data, true)\n  \n  # Customize system prompt based on options\n  system_prompt = build_prompt(risk_profile, timezone)\n  \n  # Select appropriate tools based on options\n  tools = select_tools(risk_profile, include_market_data)\n  \n  # Create the flow\n  Flows.conversation(system_prompt, tools, options)\nend\n```","ref":"custom_agents.html#configuration-options"},{"type":"extras","title":"Testing Custom Agents - Creating Custom Agents","doc":"Create tests for domain-specific behavior:\n\n```elixir\ndefmodule MyApp.FinancialAdvisorTest do\n  use ExUnit.Case\n  \n  test \"provides retirement planning advice\" do\n    {flow, state} = MyApp.FinancialAdvisor.create_agent()\n    signal = LLMAgent.Signals.user_message(\"I'm 30 years old and want to retire at 65. How much should I save?\")\n    \n    {:ok, result, _new_state} = AgentForge.process(flow, state, signal)\n    \n    assert result.type == :response\n    assert String.contains?(result.data, \"retirement\")\n    assert String.contains?(result.data, \"save\")\n  end\n  \n  test \"uses retirement calculator tool\" do\n    {flow, state} = MyApp.FinancialAdvisor.create_agent()\n    signal = LLMAgent.Signals.user_message(\"Calculate how much I need to retire if I'm currently 30, want to retire at 65, and need $50,000 per year.\")\n    \n    {:ok, tool_call, new_state} = AgentForge.process(flow, state, signal)\n    \n    assert tool_call.type == :tool_call\n    assert tool_call.data.name == \"retirement_calculator\"\n  end\nend\n```","ref":"custom_agents.html#testing-custom-agents"},{"type":"extras","title":"Real-World Examples - Creating Custom Agents","doc":"","ref":"custom_agents.html#real-world-examples"},{"type":"extras","title":"Educational Tutor - Creating Custom Agents","doc":"```elixir\ndefmodule MyApp.MathTutor do\n  alias LLMAgent.{Flows, Signals}\n  \n  def create_tutor(subject \\\\ :algebra, difficulty \\\\ :intermediate) do\n    # Build appropriate system prompt for subject and difficulty\n    system_prompt = \"\"\"\n    You are a math tutor specializing in #{subject} at the #{difficulty} level.\n    Guide students through problems step-by-step, don't just give answers.\n    Ask questions to check understanding and provide hints when students are stuck.\n    \"\"\"\n    \n    # Create appropriate tools\n    tools = [\n      %{\n        name: \"equation_solver\",\n        description: \"Solve mathematical equations. Parameter: equation\",\n        execute: &MyApp.MathTools.solve_equation/1\n      },\n      %{\n        name: \"plot_function\",\n        description: \"Create a text representation of a function graph. Parameter: function\",\n        execute: &MyApp.MathTools.plot_function/1\n      }\n    ]\n    \n    # Create the flow\n    Flows.conversation(system_prompt, tools)\n  end\nend\n```","ref":"custom_agents.html#educational-tutor"},{"type":"extras","title":"Customer Support Agent - Creating Custom Agents","doc":"```elixir\ndefmodule MyApp.SupportAgent do\n  alias LLMAgent.{Flows, Signals}\n  \n  def create_agent(product, knowledge_base) do\n    # Build system prompt with product details\n    system_prompt = \"\"\"\n    You are a customer support agent for #{product}.\n    Use the knowledge base to answer questions accurately.\n    For technical issues, ask for relevant details like error messages.\n    For billing or account issues, direct customers to appropriate resources.\n    \"\"\"\n    \n    # Create support-specific tools\n    tools = [\n      %{\n        name: \"search_knowledge_base\",\n        description: \"Search the knowledge base for articles. Parameter: query\",\n        execute: fn args -> MyApp.Support.search_kb(knowledge_base, args[\"query\"]) end\n      },\n      %{\n        name: \"create_ticket\",\n        description: \"Create a support ticket. Parameters: issue, severity, description\",\n        execute: &MyApp.Support.create_ticket/1\n      }\n    ]\n    \n    # Create the flow\n    Flows.conversation(system_prompt, tools)\n  end\nend\n```","ref":"custom_agents.html#customer-support-agent"},{"type":"extras","title":"Next Steps - Creating Custom Agents","doc":"- Explore [advanced usage patterns](advanced_use.html)\n- Learn about [LLM provider integration](provider_guide.html)\n- See the [architecture overview](architecture.html) for more extension points","ref":"custom_agents.html#next-steps"},{"type":"extras","title":"Getting Started with LLMAgent","doc":"# Getting Started with LLMAgent\n\nLLMAgent is an abstraction library built on top of AgentForge that specializes in LLM-powered agent interactions. This guide will help you get started with LLMAgent quickly.","ref":"getting_started.html"},{"type":"extras","title":"Installation - Getting Started with LLMAgent","doc":"Add LLMAgent to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:llm_agent, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"getting_started.html#installation"},{"type":"extras","title":"Configuration - Getting Started with LLMAgent","doc":"Configure your LLM provider API keys in your config:\n\n```elixir\n# config/config.exs\nconfig :llm_agent, :openai,\n  api_key: System.get_env(\"OPENAI_API_KEY\"),\n  organization_id: System.get_env(\"OPENAI_ORGANIZATION_ID\")\n\nconfig :llm_agent, :anthropic,\n  api_key: System.get_env(\"ANTHROPIC_API_KEY\")\n```","ref":"getting_started.html#configuration"},{"type":"extras","title":"Creating Your First Agent - Getting Started with LLMAgent","doc":"Creating a basic question-answering agent requires only a few lines of code:\n\n```elixir\n# Create a simple Q&A agent with a system prompt\n{flow, state} = LLMAgent.Flows.conversation(\n  \"You are a helpful assistant that answers questions concisely.\"\n)\n\n# Process a user message\nmessage = \"What is Elixir?\"\n{:ok, result, new_state} = AgentForge.process(flow, state, LLMAgent.Signals.user_message(message))\n\n# Extract the assistant response\nresponse = result.data\nIO.puts(\"Assistant: #{response}\")\n```","ref":"getting_started.html#creating-your-first-agent"},{"type":"extras","title":"Adding Tools - Getting Started with LLMAgent","doc":"Agents become more powerful when they can use tools to access external information or perform actions:\n\n```elixir\n# Define some tools\ntools = [\n  %{\n    name: \"calculator\",\n    description: \"Perform mathematical calculations\",\n    execute: fn args -> \n      expr = args[\"expression\"]\n      {:ok, result} = Code.eval_string(expr)\n      %{result: result}\n    end\n  },\n  %{\n    name: \"current_time\",\n    description: \"Get the current time\",\n    execute: fn _args -> \n      %{time: DateTime.utc_now() |> DateTime.to_string()}\n    end\n  }\n]\n\n# Create an agent with tools\n{flow, state} = LLMAgent.Flows.conversation(\n  \"You are a helpful assistant that can perform calculations and tell the time.\",\n  tools\n)\n\n# Process a message that might require a tool\nmessage = \"What's 22 + 20?\"\n{:ok, result, new_state} = AgentForge.process(flow, state, LLMAgent.Signals.user_message(message))\n```","ref":"getting_started.html#adding-tools"},{"type":"extras","title":"Next Steps - Getting Started with LLMAgent","doc":"- Learn about the [architecture](architecture.html) of LLMAgent\n- Discover how to integrate [custom tools](tool_integration.html)\n- Explore [domain-specific agents](custom_agents.html)","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Tool Integration","doc":"# Tool Integration\n\nTools extend an LLM agent's capabilities by allowing it to perform actions and access information outside of its training data. This guide explains how to create, register, and manage tools in the LLMAgent library.","ref":"tool_integration.html"},{"type":"extras","title":"Tool Architecture - Tool Integration","doc":"In LLMAgent, tools follow a simple structure:\n\n1. **Name**: A unique identifier for the tool\n2. **Description**: Explains what the tool does and how to use it\n3. **Execute Function**: A function that implements the tool's behavior\n\nTools are registered with AgentForge and are available for the LLM to use during a conversation.","ref":"tool_integration.html#tool-architecture"},{"type":"extras","title":"Defining Tools - Tool Integration","doc":"A tool is defined as a map with the following structure:\n\n```elixir\ntool = %{\n  name: \"weather_api\",\n  description: \"Get the current weather for a location. Requires a 'location' parameter.\",\n  execute: fn args ->\n    location = Map.get(args, \"location\", \"\")\n    # API call logic here\n    %{temperature: 22, conditions: \"sunny\", location: location}\n  end\n}\n```\n\nThe `execute` function takes a map of arguments and returns the tool's result. The function should be designed to handle errors gracefully and return structured data.","ref":"tool_integration.html#defining-tools"},{"type":"extras","title":"Registering Tools - Tool Integration","doc":"Tools are registered when creating a conversation flow:\n\n```elixir\ntools = [\n  %{name: \"calculator\", description: \"...\", execute: &MyApp.Tools.calculate/1},\n  %{name: \"weather\", description: \"...\", execute: &MyApp.Tools.get_weather/1}\n]\n\n{flow, state} = LLMAgent.Flows.conversation(\"You are a helpful assistant.\", tools)\n```","ref":"tool_integration.html#registering-tools"},{"type":"extras","title":"Tool Handler Mechanism - Tool Integration","doc":"When the LLM decides to use a tool, it generates a `:tool_call` signal. The `tool_handler` function in `LLMAgent.Handlers` processes this signal:\n\n1. Extracts the tool name and arguments\n2. Looks up the tool by name\n3. Executes the tool function\n4. Creates a `:tool_result` signal with the result\n5. Adds the tool call and result to the conversation history\n\nHere's a simplified version of how the tool handler works:\n\n```elixir\ndef tool_handler(%{type: :tool_call, data: %{name: tool_name, args: tool_args}}, state) do\n  case AgentForge.Tools.get(tool_name) do\n    {:ok, tool_fn} ->\n      try do\n        result = tool_fn.(tool_args)\n        result_signal = Signals.tool_result(tool_name, result)\n        new_state = Store.add_tool_call(state, tool_name, tool_args, result)\n        {{:emit, result_signal}, new_state}\n      rescue\n        e -> \n          error_signal = Signals.error(Exception.message(e), tool_name)\n          {{:emit, error_signal}, state}\n      end\n\n    {:error, reason} ->\n      error_signal = Signals.error(\"Tool not found: #{reason}\", tool_name)\n      {{:emit, error_signal}, state}\n  end\nend\n```","ref":"tool_integration.html#tool-handler-mechanism"},{"type":"extras","title":"Tool Result Processing - Tool Integration","doc":"After a tool is executed, the result is processed by the `tool_result_handler`:\n\n1. The tool result is added to the conversation history\n2. A new `:thinking` signal is generated\n3. The LLM is called again to process the tool result\n\nThis allows the LLM to use tool results to inform its next actions.","ref":"tool_integration.html#tool-result-processing"},{"type":"extras","title":"Example Tools - Tool Integration","doc":"","ref":"tool_integration.html#example-tools"},{"type":"extras","title":"Calculator Tool - Tool Integration","doc":"```elixir\ncalculator_tool = %{\n  name: \"calculator\",\n  description: \"Perform mathematical calculations. Accepts an 'expression' parameter.\",\n  execute: fn args ->\n    expr = Map.get(args, \"expression\", \"\")\n    case Code.eval_string(expr) do\n      {result, _} -> %{result: result}\n      _ -> %{error: \"Invalid expression\"}\n    end\n  end\n}\n```","ref":"tool_integration.html#calculator-tool"},{"type":"extras","title":"Web Search Tool - Tool Integration","doc":"```elixir\nsearch_tool = %{\n  name: \"web_search\",\n  description: \"Search the web for information. Requires a 'query' parameter.\",\n  execute: fn args ->\n    query = Map.get(args, \"query\", \"\")\n    case MyApp.SearchClient.search(query) do\n      {:ok, results} -> %{results: results}\n      {:error, reason} -> %{error: reason}\n    end\n  end\n}\n```","ref":"tool_integration.html#web-search-tool"},{"type":"extras","title":"Database Tool - Tool Integration","doc":"```elixir\ndb_tool = %{\n  name: \"database_query\",\n  description: \"Query a database. Requires a 'query' parameter.\",\n  execute: fn args ->\n    query = Map.get(args, \"query\", \"\")\n    case Repo.query(query) do\n      {:ok, results} -> %{results: results}\n      {:error, reason} -> %{error: reason}\n    end\n  end\n}\n```","ref":"tool_integration.html#database-tool"},{"type":"extras","title":"Best Practices - Tool Integration","doc":"","ref":"tool_integration.html#best-practices"},{"type":"extras","title":"Security Considerations - Tool Integration","doc":"Be careful with tools that:\n- Execute arbitrary code\n- Make external API calls\n- Access sensitive data\n- Modify system state\n\nAlways validate and sanitize inputs, especially when the tool has side effects.","ref":"tool_integration.html#security-considerations"},{"type":"extras","title":"Error Handling - Tool Integration","doc":"Tools should handle errors gracefully and provide useful error messages:\n\n```elixir\ndef execute(args) do\n  try do\n    # Tool logic\n    %{result: result}\n  rescue\n    e in HTTPError -> %{error: \"API error: #{e.message}\"}\n    e in Timeout -> %{error: \"Request timed out\"}\n    _ -> %{error: \"Unknown error occurred\"}\n  end\nend\n```","ref":"tool_integration.html#error-handling"},{"type":"extras","title":"Structured Results - Tool Integration","doc":"Return structured data that's easy for the LLM to understand:\n\n```elixir\n# Good\n%{\n  weather: %{\n    temperature: 22,\n    conditions: \"sunny\",\n    humidity: 65\n  },\n  location: \"New York\"\n}\n\n# Not as good\n\"Weather in New York: 22°C, sunny, 65% humidity\"\n```","ref":"tool_integration.html#structured-results"},{"type":"extras","title":"Tool Description - Tool Integration","doc":"Write clear, detailed descriptions that explain:\n- What the tool does\n- What parameters it requires\n- What format the parameters should be in\n- What the tool will return\n\nGood descriptions help the LLM use tools correctly.","ref":"tool_integration.html#tool-description"},{"type":"extras","title":"Next Steps - Tool Integration","doc":"- Learn about [custom agents](custom_agents.html)\n- Explore [LLM provider integration](provider_guide.html)\n- See [advanced usage patterns](advanced_use.html)","ref":"tool_integration.html#next-steps"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}